/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.atuscriptrecord;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.scriptcase.api.atuscriptrecord.service.AtuGlobalVariableApiService;
import net.northking.cctp.scriptcase.dto.atuscriptrecord.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 全局变量表 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "全局变量表")
@RequestMapping("/v1/AtuGlobalVariables")
@RestController
public class AtuGlobalVariableRestfulCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(AtuGlobalVariableRestfulCtrl.class);

  @Autowired
  private AtuGlobalVariableApiService apiService;

  /**
   * 分页查询 全局变量表 列表
   * 参数 系统id+变量类型
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：全局变量表")
  @PostMapping(value = "/paging/query/AtuGlobalVariables",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<AtuGlobalVariableDetailDto>> pagingQuery(
          @RequestBody @Validated QueryByPage<AtuGlobalVariableQueryDto> queryByPage)
  {
    logger.info("分页查询列表：全局变量表接口开始执行");
    ResultWrapper<Pagination<AtuGlobalVariableDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<AtuGlobalVariableDetailDto> page = apiService.pagingQuery(queryByPage);
    logger.info("分页查询列表：全局变量表接口结束执行");
    return wrapper.success(page);
  }

  /**
   * Restful接口-post： 新增全局变量表
   *
   * @param inputDto         接口数据传输对象
   * @return 主键
   */
  @ApiOperation(value = "新增：全局变量表")
  @PostMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<String> add(
      @RequestBody @Validated AtuGlobalVariableAddDto inputDto)
  {
    logger.info("新增：全局变量表接口开始执行");
    ResultWrapper<String> wrapper = apiService.add(inputDto);
    logger.info("新增：全局变量表接口结束执行");
    return wrapper;
  }

  /**
   * Restful接口-put： 更新全局变量表
   * id+默认值+说明
   * @param inputDto         接口数据传输对象
   * @return 修改的记录数量
   */
  @ApiOperation(value = "更新：全局变量表")
  @PutMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> update(
      @RequestBody @Validated AtuGlobalVariableUpdateDto inputDto) throws IllegalAccessException
  {
    logger.info("更新：全局变量表接口开始执行");
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.updateByPK(inputDto);
    logger.info("更新：全局变量表接口结束执行");
    return wrapper.success(count);
  }

  /**
   * Restful接口-delete： 删除全局变量表
   * 参数 主键
   * @param uuid       主键
   * @return 删除的记录数量
   */
  @ApiOperation(value = "删除：全局变量表")
  @DeleteMapping(
      value = "/{uuid}",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteByPK(
      @PathVariable("uuid") String uuid)
  {
    logger.info("删除：全局变量表接口开始执行");
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    // 判断该变量有没被脚本引用
    Integer count = apiService.deleteByPK(uuid);
    logger.info("删除：全局变量表接口结束执行");
    return wrapper.success(count);
  }

}
