/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.atuscriptrecord.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.scriptcase.db.entity.AtuGlobalVariable;
import net.northking.cctp.scriptcase.dto.atuscriptrecord.*;
import net.northking.cctp.scriptcase.pub.variable.dto.VariableDto;

import java.util.List;

/**
 * 全局变量表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuGlobalVariableApiService extends ExcelService
{
  /**
   * 查询全局变量表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AtuGlobalVariableDetailDto> pagingQuery(QueryByPage<AtuGlobalVariableQueryDto> queryByPage);

  /**
   * 新增全局变量表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  ResultWrapper<String> add(AtuGlobalVariableAddDto dto);

  /**
   * 更新全局变量表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuGlobalVariableUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取全局变量表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AtuGlobalVariableDetailDto findByPK(String uuid);

  /**
   * 删除全局变量表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<AtuGlobalVariableAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<AtuGlobalVariableUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  AtuGlobalVariable queryVariable(AtuGlobalVariable var);

  /**
   * 更新全局变量值
   *
   * @param update
   * @return
   */
    int updateGlobal(VariableDto update);
}
