/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.testbug.api.service.BugManagementApiService;
import net.northking.cctp.testbug.dto.BugManagementAddDto;
import net.northking.cctp.testbug.dto.BugManagementUpdateDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 缺陷管理 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "缺陷管理")
@RequestMapping("/v1/batch/BugManagements")
@RestController
public class BugManagementBatchCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(BugManagementBatchCtrl.class);

  @Autowired
  private BugManagementApiService apiService;

  @Autowired
  private RedisTemplate redisTemplate;

  /**
   * Restful接口-post： 批量新增缺陷管理
   *
   * @param batch 集合
   * @return 主键集合
   */
  @ApiOperation(value = "批量新增：缺陷管理")
  @PostMapping(
      value = "/insert",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<String>> addBatch(
      @RequestBody @Validated List<BugManagementAddDto> batch)
  {
    ResultWrapper<List<String>> wrapper = new ResultWrapper<>();
    List<String> result = this.apiService.addBatch(batch);
    return wrapper.success(result);
  }

  /**
   * Restful接口-post： 批量修改缺陷管理
   *
   * @param batch 集合
   * @return 成功的数量
   */
  @ApiOperation(value = "批量修改：缺陷管理")
  @PostMapping(
      value = "/update",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> updateBatch(
      @RequestBody @Validated List<BugManagementUpdateDto> batch) throws IllegalAccessException
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = this.apiService.updateBatch(batch);
    return wrapper.success(count);
  }

  /**
   * Restful接口-post： 批量删除缺陷管理
   *
   * @param batch 集合
   * @return 成功的数量
   */
  @ApiOperation(value = "批量删除：缺陷管理")
  @PostMapping(
      value = "/delete",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteBatch(
      @RequestBody @Validated List<Object> batch)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = this.apiService.deleteBatch(batch);
    return wrapper.success(count);
  }
}
