/*
 * Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
 *
 */
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.db.entity.BugLog;
import net.northking.cctp.testbug.dto.BugLogAddDto;
import net.northking.cctp.testbug.dto.BugLogDetailDto;
import net.northking.cctp.testbug.dto.BugLogQueryDto;
import net.northking.cctp.testbug.dto.BugLogUpdateDto;

import java.util.List;

/**
 * 缺陷注释表Restful逻辑接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugLogApiService extends ExcelService {
    /**
     * 查询缺陷注释表列表
     *
     * @param queryByPage 查询条件
     * @return 查询结果列表
     */
    Pagination<BugLogDetailDto> pagingQuery(QueryByPage<BugLogQueryDto> queryByPage);

    /**
     * 新增缺陷注释表
     *
     * @param dto 接口数据传输对象
     * @return 主键
     */
    String add(BugLogAddDto dto);

    /**
     * 更新缺陷注释表
     *
     * @param dto 接口数据传输对象
     * @return 修改的记录数量
     */
    Integer updateByPK(BugLogUpdateDto dto) throws IllegalAccessException;

    /**
     * 获取缺陷注释表
     *
     * @param uuid 主键
     * @return 接口数据传输对象
     */
    BugLogDetailDto findByPK(String uuid);

    /**
     * 删除缺陷注释表
     *
     * @param uuid 主键
     * @return 删除的记录数量
     */
    Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugLogAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugLogUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    /**
     * 根据bugId查询注释信息
     *
     * @param bugId
     * @return
     */
    List<BugLogDetailDto> queryByBugId(String bugId);

    void pubAdd(BugLog bugLogAddDto);
}
