/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.BasicService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.annotation.DatabaseReadOnly;
import net.northking.cctp.common.db.annotation.DatabaseWriteOnly;
import net.northking.cctp.common.db.excel.AbstractExcelService;
import net.northking.cctp.common.db.excel.ExcelHeader;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.util.UUIDUtil;

import net.northking.cctp.dw.db.service.MtCaseExecTrendChartService;
import net.northking.cctp.dw.db.entity.MtCaseExecTrendChart;

import net.northking.cctp.dw.dto.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 用例执行趋势图服务实现类
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */

@Service
@ConditionalOnMissingBean(name = "MtCaseExecTrendChartApiServiceImpl")
public class MtCaseExecTrendChartApiServiceImpl extends AbstractExcelService<MtCaseExecTrendChart> implements MtCaseExecTrendChartApiService
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(MtCaseExecTrendChartApiServiceImpl.class);

  @Autowired
  private MtCaseExecTrendChartService mtCaseExecTrendChartService;

  //= Excel 导入导出相关代码 start ==================//
  @Override
  public BasicService<MtCaseExecTrendChart> getService()
  {
    return this.mtCaseExecTrendChartService;
  }

  @Override
  public ExcelHeader[] excelHeaders()
  {
    ExcelHeader id = new ExcelHeader("主键id", "id", String.class);
    ExcelHeader tenantId = new ExcelHeader("租户id", "tenantId", String.class);
    ExcelHeader caseExecAllAmount = new ExcelHeader("执行用例总数量", "caseExecAllAmount", Integer.class);
    ExcelHeader caseExecAmount = new ExcelHeader("已执行用例数量", "caseExecAmount", Integer.class);
    ExcelHeader caseNoExecAmount = new ExcelHeader("未执行用例数量", "caseNoExecAmount", Integer.class);
    ExcelHeader createdTime = new ExcelHeader("创建时间", "createdTime", String.class);
    ExcelHeader projectId = new ExcelHeader("", "projectId", String.class);
    // TODO 定义表头及顺序
    return new ExcelHeader[]{};
  }

  /**
   * Excel导入，入库之前的处理
   * @param records 从Excel读取到的记录
   * @return 待入库的记录
   */
  @Override
  protected List<MtCaseExecTrendChart> beforeInsertBatch(List<MtCaseExecTrendChart> records)
  {
    for (MtCaseExecTrendChart record : records)
    {
      record.setId(UUIDUtil.create32UUID());
    }
    return records;
  }

  /**
   * 写入Excel之前的处理<br>
   * 数据的特殊处理在此处加工处理
   *
   * @param record 来自数据库记录
   * @return 待写入Excel的记录
   */
  @Override
  protected MtCaseExecTrendChart beforeWriteExcel(MtCaseExecTrendChart record)
  {
    return record;
  }

  //= Excel 导入导出相关代码 end =======================//

  //= 增删改查 相关代码 start ==================//
  @Override
  @DatabaseReadOnly
  public Pagination<MtCaseExecTrendChartDetailDto> pagingQuery(QueryByPage<MtCaseExecTrendChartQueryDto> queryByPage)
  {
    return this.mtCaseExecTrendChartService.pagingQuery(queryByPage);
  }

  @Override
  @DatabaseWriteOnly
  public String add(MtCaseExecTrendChartAddDto dto)
  {
    String uuid = UUIDUtil.create32UUID();
    MtCaseExecTrendChart entity = new MtCaseExecTrendChart();
    BeanUtils.copyProperties(dto, entity);
    entity.setId(uuid);
    this.mtCaseExecTrendChartService.insert(entity);
    return uuid;
  }

  @Override
  @DatabaseWriteOnly
  public Integer updateByPK(MtCaseExecTrendChartUpdateDto dto) throws IllegalAccessException
  {
    MtCaseExecTrendChart entity = new MtCaseExecTrendChart();
    BeanUtils.copyProperties(dto, entity);
    return this.mtCaseExecTrendChartService.updateByPrimaryKey(entity);
  }

  @Override
  @DatabaseReadOnly
  public MtCaseExecTrendChartDetailDto findByPK(String uuid)
  {
    MtCaseExecTrendChart example = new MtCaseExecTrendChart();
    example.setId(uuid);
    List<MtCaseExecTrendChart> entitys = this.mtCaseExecTrendChartService.query(example);
    if (entitys.isEmpty())
    {
      return null;
    }

    MtCaseExecTrendChartDetailDto dto = new MtCaseExecTrendChartDetailDto();
    BeanUtils.copyProperties(entitys.get(0), dto);
    return dto;
  }

  @Override
  @DatabaseWriteOnly
  public Integer deleteByPK(String uuid)
  {
    MtCaseExecTrendChart entity = new MtCaseExecTrendChart();
    entity.setId(uuid);
    return this.mtCaseExecTrendChartService.deleteByExample(entity);
  }

  @Override
  public List<String> addBatch(List<MtCaseExecTrendChartAddDto> batch)
  {
    List<String> result = new ArrayList<>();
    for (MtCaseExecTrendChartAddDto dto : batch)
    {
      result.add(this.add(dto));
    }
    return result;
  }

  @Override
  public Integer updateBatch(List<MtCaseExecTrendChartUpdateDto> batch) throws IllegalAccessException
  {
    int sum = 0;
    for (MtCaseExecTrendChartUpdateDto dto : batch)
    {
      sum = sum + this.updateByPK(dto);
    }
    return sum;
  }

  @Override
  public Integer deleteBatch(List<Object> batch)
  {
    return this.mtCaseExecTrendChartService.deleteByPrimaryKeys(batch);
  }

  @Override
  public List<MtCaseExecTrendChartDetailDto> listQuery(ChartDataGeneQueryDto chartDataGeneQueryDto) {
    return this.mtCaseExecTrendChartService.listQuery(chartDataGeneQueryDto);
  }

}
